<?php
/*
Plugin Name: Feedbacks and Reviews
Plugin URI: http://feedbacksplugin.wp-themes.it/
Description: Amazing Feedbacks and Reviews plugin for WordPress.
Version: 1.0
Author: Pasquale Bucci
Author URI: http://wp-themes.it/
License: GPL2
*/

/*  Copyright 2016 Pasquale Bucci (email : wpthemes.italia@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
    die;
}

/**
 * Include CMB2
 */
require_once dirname( __FILE__ ) . '/cmb2/init.php';

/**
 * Admin Enqueue scripts.
 */
function paky_feedback_admin_script() {
	//register styles
	wp_enqueue_style( 'paky-feedback-admin', plugins_url( 'assets/css/paky-feedback.css', __FILE__ ) );
	wp_enqueue_style( 'paky-feedback-bootstrap', plugins_url( 'assets/css/bootstrap.css', __FILE__ ) );
	//register scripts
	wp_enqueue_script('paky-feedback-validate-js', plugins_url('assets/js/form-validator/jquery.form-validator.min.js', __FILE__), array( 'jquery' ), '', true );
}
add_action( 'admin_enqueue_scripts', 'paky_feedback_admin_script' );

/**
 * Enqueue scripts and styles.
 */
function paky_feedback_scripts() {
    //register styles
	wp_enqueue_style( 'paky-feedback-slick', plugins_url( 'assets/css/slick.css', __FILE__ ) );
	wp_enqueue_style( 'paky-feedback-tipso', plugins_url( 'assets/css/tipso.min.css', __FILE__ ) );
	wp_enqueue_style( 'paky-feedback-bootstrap', plugins_url( 'assets/css/bootstrap.css', __FILE__ ) );
	wp_enqueue_style( 'dashicons' );
	wp_enqueue_style( 'paky-feedback', plugins_url( 'assets/css/paky-feedback.css', __FILE__ ) );
    //register scripts
	wp_enqueue_script('paky-feedback-validate-js', plugins_url('assets/js/form-validator/jquery.form-validator.min.js', __FILE__), array( 'jquery' ), '', true );
	wp_enqueue_script('paky-feedback-slick-js', plugins_url('assets/js/slick.min.js', __FILE__), array( 'jquery' ), '', true );
	wp_enqueue_script('paky-feedback-tipso-js', plugins_url('assets/js/tipso.min.js', __FILE__), array( 'jquery' ), '', true );
	wp_enqueue_script('paky-feedback-js', plugins_url('assets/js/paky-feedback.js', __FILE__), array( 'jquery' ), '', true );
}
add_action( 'wp_enqueue_scripts', 'paky_feedback_scripts' );

/**
 * Registers a Custom Post Type called Feedback
 */
function paky_feedback_post_type() {

	$labels = array(
		'name'                  => _x( 'Feedbacks', 'Post Type General Name', 'paky_feedback' ),
		'singular_name'         => _x( 'Feedback', 'Post Type Singular Name', 'paky_feedback' ),
		'menu_name'             => __( 'Feedbacks', 'paky_feedback' ),
		'name_admin_bar'        => __( 'Post Type', 'paky_feedback' ),
		'archives'              => __( 'Item Archives', 'paky_feedback' ),
		'parent_item_colon'     => __( 'Parent Item:', 'paky_feedback' ),
		'all_items'             => __( 'All Feedbacks', 'paky_feedback' ),
		'add_new_item'          => __( 'Add New Feedback', 'paky_feedback' ),
		'add_new'               => __( 'Add New Feedback', 'paky_feedback' ),
		'new_item'              => __( 'New Feedback', 'paky_feedback' ),
		'edit_item'             => __( 'Edit Feedback', 'paky_feedback' ),
		'update_item'           => __( 'Update Feedback', 'paky_feedback' ),
		'view_item'             => __( 'View Feedback', 'paky_feedback' ),
		'search_items'          => __( 'Search Feedback', 'paky_feedback' ),
		'not_found'             => __( 'Not found', 'paky_feedback' ),
		'not_found_in_trash'    => __( 'Not found in Trash', 'paky_feedback' ),
		'insert_into_item'      => __( 'Insert into item', 'paky_feedback' ),
		'uploaded_to_this_item' => __( 'Uploaded to this item', 'paky_feedback' ),
		'items_list'            => __( 'Feedbacks list', 'paky_feedback' ),
		'items_list_navigation' => __( 'Feedbacks list navigation', 'paky_feedback' ),
		'filter_items_list'     => __( 'Filter feedbacks list', 'paky_feedback' ),
	);
	$args = array(
		'label'                 => __( 'Feedback', 'paky_feedback' ),
		'description'           => __( 'Post Type Description', 'paky_feedback' ),
		'labels'                => $labels,
		'supports'              => array( 'title','editor' ),
		'hierarchical'          => false,
		'public'                => true,
		'show_ui'               => true,
		'show_in_menu'          => true,
		'menu_position'         => 5,
		'menu_icon'             => 'dashicons-megaphone',
		'show_in_admin_bar'     => true,
		'show_in_nav_menus'     => true,
		'can_export'            => true,
		'has_archive'           => false,		
		'exclude_from_search'   => true,
		'publicly_queryable'    => true,
		'capability_type'       => 'page',
	);
	register_post_type( 'paky-feedback', $args );

}
add_action( 'init', 'paky_feedback_post_type', 0 );

/**
 * Add count bubble to feedback admin menu
 */
function add_paky_feedback_bubble( $menu ) {
	$count_posts = wp_count_posts( 'paky-feedback' );
    $pending_count = $count_posts->pending;
    foreach( $menu as $menu_key => $menu_data ) {
        if( 'edit.php?post_type=paky-feedback' != $menu_data[2] )
            continue;
        $menu[$menu_key][0] .= " <span class='update-plugins count-$pending_count'><span class='plugin-count'>" . number_format_i18n($pending_count) . '</span></span>';
    }
    return $menu;
}
add_filter( 'add_menu_classes', 'add_paky_feedback_bubble');

/**
 * Register the form and fields for our front-end submission form
 */
function paky_feedback_frontend_form_register() {
	$cmb = new_cmb2_box( array(
		'id'           => 'paky-feedback-front-end-post-form',
		'object_types' => array( 'paky-feedback' ),
		'hookup'       => false,
		'save_fields'  => false,
	) );
	$cmb->add_field( array(
		'name'    => __( 'New Feedback', 'paky_feedback' ),
		'id'      => 'submitted_post_title',
		'type'    => 'hidden',
		'default' => 'paky_feedback_set_title_default',
	) );
	
	$criterions = paky_feedback_get_option( 'paky_feedback_criterion' );
	if ( ! empty( $criterions ) ) {
		
    		$cmb->add_field( array(
				'name' => $criterions,
				'id'   => 'paky_feedback_star_rating_' . preg_replace( '/\s+/', '', $criterions ),
				'type' => 'star_rating',
			) );
		
	}
		
	$cmb->add_field( array(
		'name'    => __( 'New Post Content', 'paky_feedback' ),
		'id'      => 'post_content',
		'type'    => 'textarea',
		'before' => 'paky_set_field_content_data_attr',
	) );
	$cmb->add_field( array(
		'name' => __( 'Your Name', 'paky_feedback' ),
		'id'   => 'paky_feedback_submitted_author_name',
		'type' => 'text',
		'before' => 'paky_set_field_name_data_attr',
	) );
	$cmb->add_field( array(
		'name' => __( 'Your Email', 'paky_feedback' ),
		'id'   => 'paky_feedback_submitted_author_email',
		'type' => 'text',
		'before' => 'paky_set_field_email_data_attr',
	) );
}
add_action( 'cmb2_init', 'paky_feedback_frontend_form_register' );

function paky_set_field_content_data_attr( $args, $field ) {
    $field->args['attributes']['data-validation'] = 'required';
}

function paky_set_field_name_data_attr( $args, $field ) {
    $field->args['attributes']['data-validation'] = 'required';
}

function paky_set_field_email_data_attr( $args, $field ) {
    $field->args['attributes']['data-validation'] = 'email';
}

function paky_feedback_set_title_default( $field_args, $field ) {
	return 'Feedback'. current_time( 'timestamp' );
}

function paky_feedback_admin_form_register() {
	$user_data = new_cmb2_box( array(
		'id'            => 'paky_admin_user_metabox',
		'title'         => __( 'User Data', 'paky_feedback' ),
		'object_types'  => array( 'paky-feedback', ),
	) );
	
	$criterions = paky_feedback_get_option( 'paky_feedback_criterion' );
	if ( ! empty( $criterions ) ) {
		
    		$user_data->add_field( array(
				'name' => $criterions,
				'id'   => 'paky_feedback_star_rating_' . preg_replace( '/\s+/', '', $criterions ),
				'type' => 'star_rating',
			) );
		
	}
	
	$user_data->add_field( array(
		'name'       => __( 'User Name', 'paky_feedback' ),
		'id'         => 'paky_feedback_submitted_author_name',
		'type'       => 'text',
	) );
	$user_data->add_field( array(
		'name' => __( 'User Email', 'paky_feedback' ),
		'id'   => 'paky_feedback_submitted_author_email',
		'type' => 'text',
	) );
	$admin_reply = new_cmb2_box( array(
		'id'            => 'paky_admin_reply_metabox',
		'title'         => __( 'Admin Reply', 'paky_feedback' ),
		'object_types'  => array( 'paky-feedback', ),
	) );
	$admin_reply->add_field( array(
		'name'       => __( 'Reply', 'paky_feedback' ),
		'desc' => __( 'Please include your possible reply. (Not necessarily)', 'paky_feedback' ),
		'id'         => 'paky_feedback_admin_reply',
		'type'       => 'textarea',
	) );
}
add_action( 'cmb2_admin_init', 'paky_feedback_admin_form_register' );

/**
 * Gets the front-end-post-form cmb instance
 *
 * @return CMB2 object
 */
function paky_feedback_frontend_cmb2_get() {
	// Use ID of metabox in wds_frontend_form_register
	$metabox_id = 'paky-feedback-front-end-post-form';
	// Post/object ID is not applicable since we're using this form for submission
	$object_id  = 'fake-oject-id';
	// Get CMB2 metabox object
	return cmb2_get_metabox( $metabox_id, $object_id );
}
/**
 * Handle the cmb-frontend-form shortcode
 *
 * @param  array  $atts Array of shortcode attributes
 * @return string       Form html
 */
function paky_feedback_do_frontend_form_submission_shortcode( $atts = array() ) {
	// Get CMB2 metabox object
	$cmb = paky_feedback_frontend_cmb2_get();
	// Get $cmb object_types
	$post_types = $cmb->prop( 'object_types' );
	// Current user
	$user_id = get_current_user_id();
	// Parse attributes
	$atts = shortcode_atts( array(
		'post_author' => $user_id ? $user_id : 1, // Current user, or admin
		'post_status' => 'pending',
		'post_type'   => reset( $post_types ), // Only use first object_type in array
	), $atts, 'cmb-frontend-form' );
	/*
	 * Let's add these attributes as hidden fields to our cmb form
	 * so that they will be passed through to our form submission
	 */
	foreach ( $atts as $key => $value ) {
		$cmb->add_hidden_field( array(
			'field_args'  => array(
				'id'    => "atts[$key]",
				'type'  => 'hidden',
				'default' => $value,
			),
		) );
	}
	// Initiate our output variable
	$output = '';
	// Get any submission errors
	if ( ( $error = $cmb->prop( 'submission_error' ) ) && is_wp_error( $error ) ) {
		// If there was an error with the submission, add it to our ouput.
		$output .= '<h4 class="paky-error">' . sprintf( __( 'There was an error in the submission: %s', 'paky_feedback' ), '<strong>'. $error->get_error_message() .'</strong>' ) . '</h4>';
	}
	// If the post was submitted successfully, notify the user.
	if ( isset( $_GET['post_submitted'] ) && ( $post = get_post( absint( $_GET['post_submitted'] ) ) ) ) {
		// Get submitter's name
		$name = get_post_meta( $post->ID, 'paky_feedback_submitted_author_name', 1 );
		$name = $name ? ' '. $name : '';
		// Add notice of submission to our output
		$output .= '<h4 class="paky-success">' . sprintf( __( 'Thank you%s, your feedback has been submitted and is in pending review.', 'paky_feedback' ), esc_html( $name ) ) . '</h4>';

	}
	// Get our form
	$output .= cmb2_get_metabox_form( $cmb, 'fake-oject-id', array( 'save_button' => __( 'Submit Feedback', 'paky_feedback' ) ) );
	return $output;
}
add_shortcode( 'paky-feedback-form', 'paky_feedback_do_frontend_form_submission_shortcode' );
/**
 * Handles form submission on save. Redirects if save is successful, otherwise sets an error message as a cmb property
 *
 * @return void
 */
function paky_feedback_handle_frontend_new_post_form_submission() {
	// If no form submission, bail
	if ( empty( $_POST ) || ! isset( $_POST['submit-cmb'], $_POST['object_id'] ) ) {
		return false;
	}
	// Get CMB2 metabox object
	$cmb = paky_feedback_frontend_cmb2_get();
	$post_data = array();
	// Get our shortcode attributes and set them as our initial post_data args
	if ( isset( $_POST['atts'] ) ) {
		foreach ( (array) $_POST['atts'] as $key => $value ) {
			$post_data[ $key ] = sanitize_text_field( $value );
		}
		unset( $_POST['atts'] );
	}
	// Check security nonce
	if ( ! isset( $_POST[ $cmb->nonce() ] ) || ! wp_verify_nonce( $_POST[ $cmb->nonce() ], $cmb->nonce() ) ) {
		return $cmb->prop( 'submission_error', new WP_Error( 'security_fail', __( 'Security check failed.' ) ) );
	}
	// Check title submitted
	// And that the title is not the default title
	if ( $cmb->get_field( 'submitted_post_title' )->default() == $_POST['submitted_post_title'] ) {
		return $cmb->prop( 'submission_error', new WP_Error( 'post_data_missing', __( 'Please enter a new title.', 'paky_feedback' ) ) );
	}
	/**
	 * Fetch sanitized values
	 */
	$sanitized_values = $cmb->get_sanitized_values( $_POST );
	// Set our post data arguments
	$post_data['post_title']   = $sanitized_values['submitted_post_title'];
	unset( $sanitized_values['submitted_post_title'] );
	$post_data['post_content'] = $sanitized_values['post_content'];
	unset( $sanitized_values['post_content'] );
	// Create the new post
	$new_submission_id = wp_insert_post( $post_data, true );
	// If we hit a snag, update the user
	if ( is_wp_error( $new_submission_id ) ) {
		return $cmb->prop( 'submission_error', $new_submission_id );
	}
	
	// Loop through remaining (sanitized) data, and save to post-meta
	foreach ( $sanitized_values as $key => $value ) {
		if ( is_array( $value ) ) {
			$value = array_filter( $value );
			if( ! empty( $value ) ) {
				update_post_meta( $new_submission_id, $key, $value );
			}
		} else {
			update_post_meta( $new_submission_id, $key, $value );
		}
	}
	/*
	 * Redirect back to the form page with a query variable with the new post ID.
	 * This will help double-submissions with browser refreshes
	 */
	wp_redirect( esc_url_raw( add_query_arg( 'post_submitted', $new_submission_id ) ) );
	exit;
}
add_action( 'cmb2_after_init', 'paky_feedback_handle_frontend_new_post_form_submission' );

/**
* Add custom columns in feedbacks edit page
*/
function paky_feedback_add_columns($columns) {
    return array_merge($columns,
                array( 'rating' =>esc_html__( 'Rating', 'paky_feedback' ),
                    'rating_value' => esc_html__('Rating Value', 'paky_feedback' ),
                    'user_name' => esc_html__( 'User Name', 'paky_feedback' ),
                    'user_email' => esc_html__( 'User Email', 'paky_feedback' ),
                ));
}
function paky_feedback_column( $column, $post_id ) {
    switch ( $column ) {

        case 'rating' :
			$i = $stars = $starating = 0;
			$criterions = paky_feedback_get_option( 'paky_feedback_criterion' );
			if ( ! empty( $criterions ) ) {
				
				$starating = esc_html( get_post_meta( get_the_ID(), 'paky_feedback_star_rating_' . preg_replace( '/\s+/', '', $criterions ), true ) );
					
				echo paky_feedback_display_rating( ( $starating ), 1 );
			}
			
            break;

        case 'rating_value' :
            $i = $stars = $starating = 0;
			$criterions = paky_feedback_get_option( 'paky_feedback_criterion' );
			if ( ! empty( $criterions ) ) {
				
				$starating = esc_html( get_post_meta( get_the_ID(), 'paky_feedback_star_rating_' . preg_replace( '/\s+/', '', $criterions ), true ) );
				if ( $starating == '' ) {
					$starating = '0';
				}	
				echo number_format( $starating , 2);
			}
			
            break;

        case 'user_name' :
            echo esc_html( get_post_meta( get_the_ID(), 'paky_feedback_submitted_author_name', true ) );
            break;

        case 'user_email' :
            echo esc_html( get_post_meta( get_the_ID(), 'paky_feedback_submitted_author_email', true ) );
            break;

    }
}
add_filter( 'manage_paky-feedback_posts_columns' , 'paky_feedback_add_columns' );
add_action( 'manage_paky-feedback_posts_custom_column' , 'paky_feedback_column', 10, 2 );

/**
 * Display rating. $display 1=only stars; $display 2=only text rating;
 */
function paky_feedback_display_rating( $rating, $display=null ) {
	
	$rating_stars = 0;
	if ( $rating <= 0 ) {
		$rating_val = esc_html( __( 'N. C.', 'paky_feedback' ) );
	} elseif ( $rating > 0 && $rating <= 1.5 ) {
		$rating_val = esc_html( __( 'Insufficient', 'paky_feedback' ) );
		$rating_stars = 1;
	} elseif ( $rating > 1.5 && $rating <= 2.5 ) {
		$rating_val = esc_html( __( 'Poor', 'paky_feedback' ) );
		$rating_stars = 2;
	} elseif ( $rating > 2.5 && $rating <= 3.5 ) {
		$rating_val = esc_html( __( 'Fair', 'paky_feedback' ) );
		$rating_stars = 3;
	} elseif ( $rating > 3.5 && $rating <= 4.5 ) {
		$rating_val = esc_html( __( 'Good', 'paky_feedback' ) );
		$rating_stars = 4;
	} elseif ( $rating > 4.5 ) {
		$rating_val = esc_html( __( 'Excellent', 'paky_feedback' ) );
		$rating_stars = 5;
	}
	
	$stars_container = '<section class="cmb2-star-container">';
	$x = 1;
	$total = 5;
		while( $x <= $rating_stars ) {
			$stars_container .= '<span class="dashicons dashicons-star-filled"></span>';
			$x++;
		}
		if( $rating_stars < $total ) {
			while( $rating_stars < $total ) {
				$stars_container .= '<span class="dashicons dashicons-star-empty"></span>';
				$rating_stars++;
			}
		}
	if ( 1 == $display ) {
		$stars_container .= '</section>';
	} elseif ( 2 == $display ) {
		$stars_container = '<section class="cmb2-star-container"><span class="valutation">'.$rating_val.'</span></section>';
	} else {
		$stars_container .= '<span class="valutation">( '.$rating_val.' )</span></section>';
	}
	
	return $stars_container;

}

/**
 * Plugin action links;
 */
function paky_feedback_action_links( $links ) {
   	$links[] = '<a href="http://feedbacksplugin.wp-themes.it/documentation/" target="_blank">Documentation</a>';
	$links[] = '<a href="http://feedbacksplugin.wp-themes.it/pricing-plugins/" target="_blank">Pricing</a>';
   	return $links;
}
add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'paky_feedback_action_links' );

/**
 * Include modules
 */
require_once plugin_dir_path( __FILE__ ) . 'inc/paky-feedback-options.php';
require_once plugin_dir_path( __FILE__ ) . 'inc/star-rating.php';
require_once plugin_dir_path( __FILE__ ) . 'inc/view-shortcodes.php';

?>
