<?php
/**
 * CMB2 Theme Options
 * @version 0.1.0
 */
class paky_feedback_Admin {
	/**
 	 * Option key, and option page slug
 	 * @var string
 	 */
	private $key = 'paky_feedback_options';
	/**
 	 * Options page metabox id
 	 * @var string
 	 */
	private $metabox_id = 'paky_feedback_option_metabox';
	/**
	 * Options Page title
	 * @var string
	 */
	protected $title = '';
	/**
	 * Options Page hook
	 * @var string
	 */
	protected $options_page = '';
	/**
	 * Holds an instance of the object
	 *
	 * @var Myprefix_Admin
	 **/
	private static $instance = null;
	/**
	 * Constructor
	 * @since 0.1.0
	 */
	private function __construct() {
		// Set our title
		$this->title = __( 'Paky Feedback Options', 'paky_feedback' );
	}
	/**
	 * Returns the running object
	 *
	 * @return paky_feedback_Admin
	 **/
	public static function get_instance() {
		if( is_null( self::$instance ) ) {
			self::$instance = new paky_feedback_Admin();
			self::$instance->hooks();
		}
		return self::$instance;
	}
	/**
	 * Initiate our hooks
	 * @since 0.1.0
	 */
	public function hooks() {
		add_action( 'admin_init', array( $this, 'init' ) );
		add_action( 'admin_menu', array( $this, 'add_options_page' ) );
		add_action( 'cmb2_admin_init', array( $this, 'add_options_page_metabox' ) );
	}
	/**
	 * Register our setting to WP
	 * @since  0.1.0
	 */
	public function init() {
		register_setting( $this->key, $this->key );
	}
	/**
	 * Add menu options page
	 * @since 0.1.0
	 */
	public function add_options_page() {
		//$this->options_page = add_menu_page( $this->title, $this->title, 'manage_options', $this->key, array( $this, 'admin_page_display' ) );
		$this->options_page = add_submenu_page('edit.php?post_type=paky-feedback', __( 'Feedbacks and Reviews Settings', 'paky_feedback' ), __( 'Settings', 'paky_feedback' ), 'edit_posts', basename(__FILE__), array( $this, 'admin_page_display' ));
		// Include CMB CSS in the head to avoid FOUC
		add_action( "admin_print_styles-{$this->options_page}", array( 'CMB2_hookup', 'enqueue_cmb_css' ) );
	}
	/**
	 * Admin page markup. Mostly handled by CMB2
	 * @since  0.1.0
	 */
	public function admin_page_display() {
		?>
		<div class="wrap cmb2-options-page <?php echo $this->key; ?>">
			<h2><?php echo esc_html( get_admin_page_title() ); ?></h2>
			
			<div style="border-left: 4px solid #43609C; padding: 20px; background: white; margin: 20px 0; box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.1); width: 70%;">
				<h3>Feedbacks and Reviews Premium</h3>
				<p>This plugin also has a Premium version. <a href="http://feedbacksplugin.wp-themes.it/pricing-plugins/">Compare Free and Premium plugin here!</a></p>
			</div>
			<?php cmb2_metabox_form( $this->metabox_id, $this->key ); ?>
		</div>
		<?php
	}
	/**
	 * Add the options metabox to the array of metaboxes
	 * @since  0.1.0
	 */
	function add_options_page_metabox() {
		// hook in our save notices
		add_action( "cmb2_save_options-page_fields_{$this->metabox_id}", array( $this, 'settings_notices' ), 10, 2 );
		$cmb = new_cmb2_box( array(
			'id'         => $this->metabox_id,
			'hookup'     => false,
			'cmb_styles' => false,
			'show_on'    => array(
				// These are important, don't remove
				'key'   => 'options-page',
				'value' => array( $this->key, )
			),
		) );
		
		$cmb->add_field( array(
    		'name' => __( 'General Settings', 'paky_feedback' ),
    		'desc' => __( 'General Settings for Feedbacks and Reviews Plugin.', 'paky_feedback' ),
    		'type' => 'title',
    		'id'   => 'paky_feedback_title1',
		) );
		
		$cmb->add_field( array(
			'name' => __( 'Feedback Form page URL', 'paky_feedback' ),
			//'desc' => __( 'field description (optional)', 'paky_feedback' ),
			'id'   => 'paky_feedback_form_page_url',
			'type' => 'text',
		) );
		
		$cmb->add_field( array(
			'name' => __( 'Feedbacks per page', 'paky_feedback' ),
			'desc' => __( 'Max number of feedbacks in the list feedbacks page', 'paky_feedback' ),
			'id'   => 'paky_feedback_max_list',
			'default' => 10,
			'type' => 'text',
			'attributes' => array(
				'type' => 'number',
				'pattern' => '\d*',
			),
		) );
		
		$cmb->add_field( array(
    		'name' => __( 'Rating Criterion', 'paky_feedback' ),
    		//'desc' => __( 'This is a title description', 'paky_feedback' ),
    		'type' => 'title',
    		'id'   => 'paky_feedback_title2',
		) );
		
		$cmb->add_field( array(
			'name' => __( 'Criterion', 'paky_feedback' ),
			'id'   => 'paky_feedback_criterion',
			'type' => 'text',
			
		) );
		
	}
	/**
	 * Register settings notices for display
	 *
	 * @since  0.1.0
	 * @param  int   $object_id Option key
	 * @param  array $updated   Array of updated fields
	 * @return void
	 */
	public function settings_notices( $object_id, $updated ) {
		if ( $object_id !== $this->key || empty( $updated ) ) {
			return;
		}
		add_settings_error( $this->key . '-notices', '', __( 'Settings updated.', 'paky_feedback' ), 'updated' );
		settings_errors( $this->key . '-notices' );
	}
	/**
	 * Public getter method for retrieving protected/private variables
	 * @since  0.1.0
	 * @param  string  $field Field to retrieve
	 * @return mixed          Field value or exception is thrown
	 */
	public function __get( $field ) {
		// Allowed fields to retrieve
		if ( in_array( $field, array( 'key', 'metabox_id', 'title', 'options_page' ), true ) ) {
			return $this->{$field};
		}
		throw new Exception( 'Invalid property: ' . $field );
	}
}
/**
 * Helper function to get/return the Myprefix_Admin object
 * @since  0.1.0
 * @return Myprefix_Admin object
 */
function paky_feedback_admin() {
	return paky_feedback_Admin::get_instance();
}
/**
 * Wrapper function around cmb2_get_option
 * @since  0.1.0
 * @param  string  $key Options array key
 * @return mixed        Option value
 */
function paky_feedback_get_option( $key = '' ) {
	return cmb2_get_option( paky_feedback_admin()->key, $key );
}

// Get it started
paky_feedback_admin();
